from playwright.sync_api import Page, expect

from shiny.playwright import controller
from shiny.playwright.expect._internal import _expect_nav_to_have_header_footer
from shiny.run import ShinyAppProc


def test_page_navbar_header_footer_fixed_top(
    page: Page, local_app: ShinyAppProc
) -> None:
    page.goto(local_app.url)

    page_navbar_header_footer_fixed_top = controller.PageNavbar(
        page, "page_navbar_header_footer_fixed_top"
    )
    page_navbar_header_footer_fixed_top.expect_position("fixed-top")
    _expect_nav_to_have_header_footer(
        page_navbar_header_footer_fixed_top.get_loc_active_content()
        .locator("..")
        .locator(".."),
        "page_navbar_header",
        "page_navbar_footer",
    )

    # assert header and footer contents
    expect(page.locator("#page_navbar_header")).to_have_text("Header")
    expect(page.locator("#page_navbar_footer")).to_have_text("Footer")

    # not working as expected since not showing on app
    page_navbar_header_footer_fixed_top.expect_fluid(False)



import pandas as pd

from shiny.express import render, ui

green_styles = [
    {
        "rows": [2, 4],
        "cols": [2, 4],
        "style": {
            "background-color": "mediumspringgreen",
            "width": "300px",
            "height": "100px",
        },
    }
]

n = 6
df = pd.DataFrame(
    {
        "a": range(n),
        "b": range(n, n * 2),
        "c": range(n * 2, n * 3),
        "d": range(n * 3, n * 4),
        "e": range(n * 4, n * 5),
    }
)

ui.h2("Data Frame with Styles applied to 4 cells")


@render.data_frame
def my_df():
    return render.DataGrid(
        df,
        styles=green_styles,
    )


ui.hr()

ui.h2("Custom styles applied to all cells within a data frame ", ui.HTML("&#128075;"))

ui.tags.style(
    ui.HTML(
        """
    .posit-bg {
        background-color: #242a26 ;
    }
    .posit-blue-bg {
        background-color: #447099 ;
    }
    .posit-orange-bg {
        background-color: #ED642F ;
    }
    """
    )
)

hi_styles = [
    {
        # No `rows` or `cols` means apply to all cells
        "class": "posit-bg",
        "style": {
            "border": "transparent",
            "color": "transparent",
        },
    },
    {
        "rows": [3],
        "cols": [2],
        "class": "posit-blue-bg",
        "style": {
            "width": "100px",
            "height": "75px",
        },
    },
    {
        "cols": [1, 3, 5],
        "class": "posit-blue-bg",
    },
    {
        "cols": [7],
        "rows": [0, 1, 2, 3, 5],
        "class": "posit-orange-bg",
    },
]

n = 7
hi_pd = pd.DataFrame(
    {
        "a": range(n),
        "b": range(n, n * 2),
        "c": range(n * 2, n * 3),
        "d": range(n * 3, n * 4),
        "e": range(n * 4, n * 5),
        "f": range(n * 5, n * 6),
        "g": range(n * 6, n * 7),
        "h": range(n * 7, n * 8),
        "i": range(n * 8, n * 9),
    }
)


@render.data_frame
def hi_df():
    return render.DataGrid(
        hi_pd,
        styles=hi_styles,
    )



from playwright.sync_api import Page

from shiny.playwright import controller
from shiny.run import ShinyAppProc


def test_checkbox_group_kitchen(page: Page, local_app: ShinyAppProc) -> None:
    page.goto(local_app.url)

    default = controller.InputRadioButtons(page, "default")
    default.expect_label("Default radio buttons")
    default.expect_selected("Option A")
    controller.OutputCode(page, "default_txt").expect_value("Option A")

    selected = controller.InputRadioButtons(page, "selected")
    selected.expect_selected("Option B")
    controller.OutputCode(page, "selected_txt").expect_value("Option B")

    width = controller.InputRadioButtons(page, "width")
    width.expect_width("30px")
    width.expect_inline(False)

    inline = controller.InputRadioButtons(page, "inline")
    inline.expect_inline(True)

    choices_dict = controller.InputRadioButtons(page, "choices_dict")
    choices_dict.expect_selected("a")
    controller.OutputCode(page, "choices_dict_txt").expect_value("a")
    choices_dict.set("c")
    choices_dict.expect_selected("c")
    controller.OutputCode(page, "choices_dict_txt").expect_value("c")



from shiny import reactive
from shiny.express import input, ui

ui.input_radio_buttons("controller1", "Controller1", ["1", "2", "3"], selected="2")

with ui.navset_hidden(
    id="hidden_tabs1",
    header=ui.tags.div("Navset_hidden_header", id="navset_hidden_header1"),
    footer=ui.tags.div("Navset_hidden_footer", id="navset_hidden_footer1"),
    selected="panel2",
):
    with ui.nav_panel(None, value="panel1"):
        "Panel 1 content"
    with ui.nav_panel(None, value="panel2"):
        "Panel 2 content"
    with ui.nav_panel(None, value="panel3"):
        "Panel 3 content"

ui.markdown("-----")
ui.input_radio_buttons("controller2", "Controller2", ["4", "5", "6"])

with ui.navset_hidden(id="hidden_tabs2"):
    with ui.nav_panel(None, value="panel4"):
        "Panel 4 content"
    with ui.nav_panel(None, value="panel5"):
        "Panel 5 content"
    with ui.nav_panel(None, value="panel6"):
        "Panel 6 content"


@reactive.effect
@reactive.event(input.controller1)
def _():
    ui.update_navs("hidden_tabs1", selected="panel" + str(input.controller1()))


@reactive.effect
@reactive.event(input.controller2)
def _():
    ui.update_navs("hidden_tabs2", selected="panel" + str(input.controller2()))



from data import adjectives, animals, dark_color, light_color

from shiny import App, Inputs, Outputs, Session, reactive, render, ui

app_ui = ui.page_fixed(
    ui.h1("Toggle Sidebars"),
    ui.div(
        ui.input_action_button("open_all", "Show all", class_="me-1 mb-1"),
        ui.input_action_button("close_all", "Close all", class_="mb-1"),
        ui.tags.br(),
        ui.input_action_button("toggle_outer", "Toggle outer", class_="me-1"),
        ui.input_action_button("toggle_inner", "Toggle inner"),
        class_="my-2",
    ),
    ui.layout_sidebar(
        ui.sidebar(
            "Outer Sidebar",
            ui.input_select(
                "adjective",
                "Adjective",
                choices=adjectives,
                selected=adjectives[0],
            ),
            id="sidebar_outer",
            width=200,
            bg=dark_color,
            fg="white",
            open="desktop",
            max_height_mobile="300px",
        ),
        ui.layout_sidebar(
            ui.sidebar(
                "Inner Sidebar",
                ui.input_select(
                    "animal",
                    "Animal",
                    choices=animals,
                    selected=animals[0],
                ),
                id="sidebar_inner",
                # width=200,
                bg=light_color,
                open="desktop",
            ),
            ui.h2("Sidebar Layout"),
            ui.output_ui("ui_content", tabindex=0),
            height=300,
            id="main_inner",
            border=False,
            border_radius=False,
        ),
        id="main_outer",
        class_="p-0",
    ),
    title="bslib | Tests | Dynamic Sidebars",
)


def server(input: Inputs, output: Outputs, session: Session) -> None:
    @render.ui
    def ui_content():
        return f"Hello, {input.adjective()} {input.animal()}!"

    @reactive.effect
    @reactive.event(input.open_all)
    def _():
        ui.update_sidebar("sidebar_inner", show=True)
        ui.update_sidebar("sidebar_outer", show=True)

    @reactive.effect
    @reactive.event(input.close_all)
    def _():
        ui.update_sidebar("sidebar_inner", show=False)
        ui.update_sidebar("sidebar_outer", show=False)

    @reactive.effect
    @reactive.event(input.toggle_inner)
    def _():
        ui.update_sidebar("sidebar_inner", show=not input.sidebar_inner())

    @reactive.effect
    @reactive.event(input.toggle_outer)
    def _():
        ui.update_sidebar("sidebar_outer", show=not input.sidebar_outer())


app = App(app_ui, server)
